/*
 * DHCP Client
 *
 * Copyright 2006 Stefan Rompf <sux@loplof.de>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * version 2 as published by the Free Software Foundation
 *
 */


#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/wait.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <time.h>
#include <string.h>


#include "dhcpprotocol.h"

static char numbuf[32];

static char *intenv(const int arg) {
  sprintf(numbuf, "%d", arg);
  return numbuf;
}

static char *timeenv(const time_t *t) {
  char *rc = ctime(t);
  rc[strlen(rc)-1] = 0;
  return rc;
}

/* limit strings forwarded to script to RFC1123
   allowed characters and '_' */
static const char *sanedomainname(const char *hn) {
  const char *n = hn;
  char c;

  while((c = *n++))
    if ((c < 'A' || c > 'Z') && (c < 'a' || c > 'z') &&
	(c < '0' || c > '9') && c != '-' && c != '_' &&
	c != '.') return "";

  return hn;
}


int pt_callscript(const struct dhcp_interface *dhcpif, dhcp_state newstate) {
  int status = -1;
  pid_t pid;
  static char dnsstr[] = "DHCP_DNS_0";

  if (!dhcpif->callscript_path) return 0;

  pid = fork();

  if (pid > 0) {
    (void)waitpid(pid, &status, 0);
  } else if (!pid) {
    int n;

    setenv("DHCP_ADDRESS", inet_ntoa(dhcpif->current_config.address), 1);
    setenv("DHCP_HOSTNAME", sanedomainname(dhcpif->current_config.hostname), 1);
    setenv("DHCP_DOMAINNAME", sanedomainname(dhcpif->current_config.domainname), 1);
    setenv("DHCP_SERVERADDR", inet_ntoa(dhcpif->current_config.dhcpd_addr), 1);
    setenv("DHCP_NETMASK", inet_ntoa(dhcpif->current_config.netmask), 1);
    setenv("DHCP_BROADCAST", inet_ntoa(dhcpif->current_config.broadcast), 1);
    setenv("DHCP_GATEWAY", inet_ntoa(dhcpif->current_config.gateway), 1);

    setenv("DHCP_DNS_NUM", intenv(dhcpif->current_config.dns_num), 1);
    for(n=0; n<dhcpif->current_config.dns_num; n++) {
      dnsstr[9] = n+48;
      setenv(dnsstr, inet_ntoa(dhcpif->current_config.dns[n]), 1);
    }

    setenv("DHCP_LEASE", intenv(dhcpif->current_config.lease), 1);
    setenv("DHCP_T1", intenv(dhcpif->current_config.t1), 1);
    setenv("DHCP_T2", intenv(dhcpif->current_config.t2), 1);

    setenv("DHCP_TO_LEASE", timeenv(&dhcpif->lease.tv_sec), 1);
    setenv("DHCP_TO_T1", timeenv(&dhcpif->t1.tv_sec), 1);
    setenv("DHCP_TO_T2", timeenv(&dhcpif->t2.tv_sec), 1);

    execl(dhcpif->callscript_path, dhcpif->callscript_path, 
	  dhcpif->ifname, dhcp_states[newstate], dhcp_states[dhcpif->state], NULL);
    _exit(-1);
  }

  return status;
}
